@@ -33,13 +33,13 @@
 (require 'gnus-util)
 (require 'message)
 
-(defcustom gnus-startup-file "~/.newsrc"
+(defcustom gnus-startup-file (nnheader-concat gnus-home-directory ".newsrc")
   "Your `.newsrc' file.
 `.newsrc-SERVER' will be used instead if that exists."
   :group 'gnus-start
   :type 'file)
 
-(defcustom gnus-init-file "~/.gnus"
+(defcustom gnus-init-file (nnheader-concat gnus-home-directory ".gnus")
   "Your Gnus elisp startup file.
 If a file with the .el or .elc suffixes exist, it will be read
 instead."
@@ -48,7 +48,7 @@ instead."
 
 (defcustom gnus-site-init-file
   (ignore-errors
-    (concat (file-name-directory 
+    (concat (file-name-directory
 	     (directory-file-name installation-directory))
 	    "site-lisp/gnus-init"))
   "The site-wide Gnus elisp startup file.
@@ -320,7 +320,7 @@ does not affect old (already subscribed) newsgroups."
 		 (const :tag "none" nil)))
 
 (defcustom gnus-modtime-botch nil
-  "*Non-nil means .newsrc should be deleted prior to save.  
+  "*Non-nil means .newsrc should be deleted prior to save.
 Its use is due to the bogus appearance that .newsrc was modified on
 disc."
   :group 'gnus-newsrc
@@ -347,7 +347,7 @@ This hook is called after Gnus is connected to the NNTP server."
   :group 'gnus-group-new
   :type 'hook)
 
-(defcustom gnus-after-getting-new-news-hook 
+(defcustom gnus-after-getting-new-news-hook
   (when (gnus-boundp 'display-time-timer)
     '(display-time-event-handler))
   "A hook run after Gnus checks for new news."
@@ -815,7 +815,7 @@ If RAWFILE is non-nil, the .newsrc file will also be read.
 If LEVEL is non-nil, the news will be set up at level LEVEL."
   (let ((init (not (and gnus-newsrc-alist gnus-active-hashtb (not rawfile)))))
 
-    (when init 
+    (when init
       ;; Clear some variables to re-initialize news information.
       (setq gnus-newsrc-alist nil
 	    gnus-active-hashtb nil)
@@ -849,8 +849,8 @@ If LEVEL is non-nil, the news will be set up at level LEVEL."
       (gnus-cache-open))
 
     ;; Possibly eval the dribble file.
-    (and init 
-	 (or gnus-use-dribble-file gnus-slave) 
+    (and init
+	 (or gnus-use-dribble-file gnus-slave)
 	 (gnus-dribble-eval-file))
 
     ;; Slave Gnusii should then clear the dribble buffer.
@@ -874,7 +874,7 @@ If LEVEL is non-nil, the news will be set up at level LEVEL."
       (gnus-find-new-newsgroups))
 
     ;; We might read in new NoCeM messages here.
-    (when (and gnus-use-nocem 
+    (when (and gnus-use-nocem
 	       (not level)
 	       (not dont-connect))
       (gnus-nocem-scan-groups))
@@ -1156,7 +1156,7 @@ the server for new groups."
       ;; Finally we enter (if needed) the list where it is supposed to
       ;; go, and change the subscription level.  If it is to be killed,
       ;; we enter it into the killed or zombie list.
-      (cond 
+      (cond
        ((>= level gnus-level-zombie)
 	;; Remove from the hash table.
 	(gnus-sethash group nil gnus-newsrc-hashtb)
@@ -1283,25 +1283,48 @@ newsgroup."
     "Alter the ACTIVE info for GROUP to reflect the articles in the cache."
     (when gnus-cache-active-hashtb
       (let ((cache-active (gnus-gethash group gnus-cache-active-hashtb)))
-	(and cache-active 
+	(and cache-active
 	     (< (car cache-active) (car active))
 	     (setcar active (car cache-active)))
 	(and cache-active
 	     (> (cdr cache-active) (cdr active))
 	     (setcdr active (cdr cache-active)))))))
 
+(defun gnus-activate-group (group &optional scan dont-check method)
+  ;; Check whether a group has been activated or not.
+  ;; If SCAN, request a scan of that group as well.
+  (let ((method (or method (inline (gnus-find-method-for-group group))))
+	active)
+    (and (inline (gnus-check-server method))
+	 ;; We escape all bugs and quit here to make it possible to
+	 ;; continue if a group is so out-there that it reports bugs
+	 ;; and stuff.
+	 (progn
+	   (and scan
+		(gnus-check-backend-function 'request-scan (car method))
+		(gnus-request-scan group method))
+	   t)
+	 (condition-case ()
+	     (inline (gnus-request-group group dont-check method))
+	   (error nil)
+	   (quit nil))
+	 (gnus-set-active group (setq active (gnus-parse-active)))
+	 ;; Return the new active info.
+	 active)))
+
 (defun gnus-get-unread-articles-in-group (info active &optional update)
   (when active
     ;; Allow the backend to update the info in the group.
-    (when (and update 
+    (when (and update
 	       (gnus-request-update-info
-		info (gnus-find-method-for-group (gnus-info-group info))))
+		info (inline (gnus-find-method-for-group
+			      (gnus-info-group info)))))
       (gnus-activate-group (gnus-info-group info) nil t))
     (let* ((range (gnus-info-read info))
 	   (num 0))
       ;; If a cache is present, we may have to alter the active info.
       (when (and gnus-use-cache info)
-	(inline (gnus-cache-possibly-alter-active 
+	(inline (gnus-cache-possibly-alter-active
 		 (gnus-info-group info) active)))
       ;; Modify the list of read articles according to what articles
       ;; are available; then tally the unread articles and add the
@@ -1400,9 +1423,10 @@ newsgroup."
       ;; newsgroup to t.  This means that Gnus thinks that there are
       ;; unread articles, but it has no idea how many.
       (if (and (setq method (gnus-info-method info))
-	       (not (gnus-server-equal
-		     gnus-select-method
-		     (setq method (gnus-server-get-method nil method))))
+	       (not (inline
+		      (gnus-server-equal
+		       gnus-select-method
+		       (setq method (gnus-server-get-method nil method)))))
 	       (not (gnus-secondary-method-p method)))
 	  ;; These groups are foreign.  Check the level.
 	  (when (<= (gnus-info-level info) foreign-level)
@@ -1462,28 +1486,6 @@ newsgroup."
       (while list
 	(gnus-sethash (car list) (pop list) gnus-killed-hashtb)))))
 
-(defun gnus-activate-group (group &optional scan dont-check method)
-  ;; Check whether a group has been activated or not.
-  ;; If SCAN, request a scan of that group as well.
-  (let ((method (or method (gnus-find-method-for-group group)))
-	active)
-    (and (gnus-check-server method)
-	 ;; We escape all bugs and quit here to make it possible to
-	 ;; continue if a group is so out-there that it reports bugs
-	 ;; and stuff.
-	 (progn
-	   (and scan
-		(gnus-check-backend-function 'request-scan (car method))
-		(gnus-request-scan group method))
-	   t)
-	 (condition-case ()
-	     (gnus-request-group group dont-check method)
-	   (error nil)
-	   (quit nil))
-	 (gnus-set-active group (setq active (gnus-parse-active)))
-	 ;; Return the new active info.
-	 active)))
-
 (defun gnus-parse-active ()
   "Parse active info in the nntp server buffer."
   (save-excursion
@@ -1552,7 +1554,7 @@ newsgroup."
 ;; Get the active file(s) from the backend(s).
 (defun gnus-read-active-file (&optional force)
   (gnus-group-set-mode-line)
-  (let ((methods 
+  (let ((methods
 	 (append
 	  (if (gnus-check-server gnus-select-method)
 	      ;; The native server is available.
@@ -1589,10 +1591,12 @@ newsgroup."
 		    (gmethod (gnus-server-get-method nil method))
 		    groups info)
 		(while (setq info (pop newsrc))
-		  (when (gnus-server-equal
-			 (gnus-find-method-for-group 
-			  (gnus-info-group info) info)
-			 gmethod)
+		  (when (inline
+			  (gnus-server-equal
+			   (inline
+			     (gnus-find-method-for-group
+			      (gnus-info-group info) info))
+			   gmethod))
 		    (push (gnus-group-real-name (gnus-info-group info))
 			  groups)))
 		(when groups
@@ -1628,7 +1632,7 @@ newsgroup."
   ;; (concat (regexp-quote "^to\\.") "\\($\\|" (regexp-quote "\\|") "\\)")
   (string-match "\\^to\\\\\\.\\($\\|\\\\|\\)"
 		gnus-ignored-newsgroups))
-  
+
 ;; Read an active file and place the results in `gnus-active-hashtb'.
 (defun gnus-active-to-gnus-format (&optional method hashtb ignore-errors)
   (unless method
@@ -1790,7 +1794,7 @@ If FORCE is non-nil, the .newsrc file is read."
 	  (gnus-newsrc-to-gnus-format)
 	  (kill-buffer (current-buffer))
 	  (gnus-message 5 "Reading %s...done" newsrc-file)))
-      
+
       ;; Convert old to new.
       (gnus-convert-old-newsrc))))
 
@@ -1874,7 +1878,7 @@ If FORCE is non-nil, the .newsrc file is read."
 	  (unless (nthcdr 3 info)
 	    (nconc info (list nil)))
 	  (gnus-info-set-marks
-	   info (list (cons 'tick (gnus-compress-sequence 
+	   info (list (cons 'tick (gnus-compress-sequence
 				   (sort (cdr m) '<) t))))))
       (setq newsrc killed)
       (while newsrc
@@ -1954,7 +1958,7 @@ If FORCE is non-nil, the .newsrc file is read."
 			   (point)))))
 	(forward-line -1))
        (symbol
-	;; Group names can be just numbers.  
+	;; Group names can be just numbers.
 	(when (numberp symbol)
 	  (setq symbol (intern (int-to-string symbol) gnus-active-hashtb)))
 	(unless (boundp symbol)
@@ -2253,7 +2257,7 @@ If FORCE is non-nil, the .newsrc file is read."
 	;; Don't write foreign groups to .newsrc.
 	(when (or (null (setq method (gnus-info-method info)))
 		  (equal method "native")
-		  (gnus-server-equal method gnus-select-method))
+		  (inline (gnus-server-equal method gnus-select-method)))
 	  (insert (gnus-info-group info)
 		  (if (> (gnus-info-level info) gnus-level-subscribed)
 		      "!" ":"))
@@ -2350,7 +2354,7 @@ If FORCE is non-nil, the .newsrc file is read."
 ;;;
 
 (defun gnus-read-all-descriptions-files ()
-  (let ((methods (cons gnus-select-method 
+  (let ((methods (cons gnus-select-method
 		       (nconc
 			(when (gnus-archive-server-wanted-p)
 			  (list "archive"))
@@ -2393,9 +2397,11 @@ If FORCE is non-nil, the .newsrc file is read."
 	    (narrow-to-region (point-min) (point)))
 	  ;; If these are groups from a foreign select method, we insert the
 	  ;; group prefix in front of the group names.
-	  (and method (not (gnus-server-equal
-			    (gnus-server-get-method nil method)
-			    (gnus-server-get-method nil gnus-select-method)))
+	  (and method (not (inline
+			     (gnus-server-equal
+			      (gnus-server-get-method nil method)
+			      (gnus-server-get-method
+			       nil gnus-select-method))))
 	       (let ((prefix (gnus-group-prefixed-name "" method)))
 		 (goto-char (point-min))
 		 (while (and (not (eobp))
@@ -2440,7 +2446,7 @@ If FORCE is non-nil, the .newsrc file is read."
 (defun gnus-set-default-directory ()
   "Set the default directory in the current buffer to `gnus-default-directory'.
 If this variable is nil, don't do anything."
-  (setq default-directory 
+  (setq default-directory
 	(if (and gnus-default-directory
 		 (file-exists-p gnus-default-directory))
 	    (file-name-as-directory (expand-file-name gnus-default-directory))
