@@ -56,7 +56,9 @@ on your system, you could say something like:
  (autoload 'nnmail-message-id "nnmail")
  (autoload 'mail-position-on-field "sendmail")
  (autoload 'message-remove-header "message")
- (autoload 'cancel-function-timers "timers"))
+ (autoload 'cancel-function-timers "timers")
+ (autoload 'gnus-point-at-eol "gnus-util")
+ (autoload 'gnus-buffer-live-p "gnus-util"))
 
 ;;; Header access macros.
 
@@ -142,7 +144,7 @@ on your system, you could say something like:
 					references chars lines xref)
   "Create a new mail header structure initialized with the parameters given."
   (vector number subject from date id references chars lines xref))
-  
+
 ;; fake message-ids: generation and detection
 
 (defvar nnheader-fake-message-id 1)
@@ -176,7 +178,7 @@ on your system, you could say something like:
 	  ;; a case (which basically was the old function) is actually
 	  ;; about twice as fast, even though it looks messier.	 You
 	  ;; can't have everything, I guess.  Speed and elegance
-	  ;; doesn't always go hand in hand.
+	  ;; don't always go hand in hand.
 	  (vector
 	   ;; Number.
 	   (if naked
@@ -209,8 +211,10 @@ on your system, you could say something like:
 	   ;; Message-ID.
 	   (progn
 	     (goto-char p)
-	     (if (search-forward "\nmessage-id: " nil t)
-		 (nnheader-header-value)
+	     (if (search-forward "\nmessage-id:" nil t)
+		 (buffer-substring
+		  (1- (or (search-forward "<" nil t) (point)))
+		  (or (search-forward ">" nil t) (point)))
 	       ;; If there was no message-id, we just fake one to make
 	       ;; subsequent routines simpler.
 	       (nnheader-generate-fake-message-id)))
@@ -281,7 +285,7 @@ on your system, you could say something like:
 
 (defun nnheader-insert-nov (header)
   (princ (mail-header-number header) (current-buffer))
-  (insert 
+  (insert
    "\t"
    (or (mail-header-subject header) "(none)") "\t"
    (or (mail-header-from header) "(nobody)") "\t"
@@ -313,7 +317,7 @@ on your system, you could say something like:
   ;; First we find the first wanted line.
   (nnheader-find-nov-line beg)
   (delete-region (point-min) (point))
-  ;; Then we find the last wanted line. 
+  ;; Then we find the last wanted line.
   (when (nnheader-find-nov-line end)
     (forward-line 1))
   (delete-region (point) (point-max)))
@@ -529,7 +533,7 @@ If FILE is t, return the buffer contents as a string."
 (defvar jka-compr-compression-info-list)
 (defvar nnheader-numerical-files
   (if (boundp 'jka-compr-compression-info-list)
-      (concat "\\([0-9]+\\)\\(" 
+      (concat "\\([0-9]+\\)\\("
 	      (mapconcat (lambda (i) (aref i 0))
 			 jka-compr-compression-info-list "\\|")
 	      "\\)?")
@@ -552,7 +556,7 @@ If FILE is t, return the buffer contents as a string."
 (defun nnheader-directory-files-safe (&rest args)
   ;; It has been reported numerous times that `directory-files'
   ;; fails with an alarming frequency on NFS mounted file systems.
-  ;; This function executes that function twice and returns 
+  ;; This function executes that function twice and returns
   ;; the longest result.
   (let ((first (apply 'directory-files args))
 	(second (apply 'directory-files args)))
@@ -579,7 +583,7 @@ If FILE is t, return the buffer contents as a string."
 (defun nnheader-translate-file-chars (file)
   (if (null nnheader-file-name-translation-alist)
       ;; No translation is necessary.
-      file 
+      file
     ;; We translate -- but only the file name.  We leave the directory
     ;; alone.
     (let* ((i 0)
@@ -627,19 +631,6 @@ without formatting."
       (apply 'insert format args))
     t))
 
-(defun nnheader-mail-file-mbox-p (file)
-  "Say whether FILE looks like an Unix mbox file."
-  (when (and (file-exists-p file)
-	     (file-readable-p file)
-	     (file-regular-p file))
-    (save-excursion
-      (nnheader-set-temp-buffer " *mail-file-mbox-p*")
-      (nnheader-insert-file-contents file)
-      (goto-char (point-min))
-      (prog1
-	  (looking-at message-unix-mail-delimiter)
-	(kill-buffer (current-buffer))))))
-
 (defun nnheader-replace-chars-in-string (string from to)
   "Replace characters in STRING from FROM to TO."
   (let ((string (substring string 0))	;Copy string.
@@ -654,12 +645,12 @@ without formatting."
 
 (defun nnheader-file-to-group (file &optional top)
   "Return a group name based on FILE and TOP."
-  (nnheader-replace-chars-in-string 
+  (nnheader-replace-chars-in-string
    (if (not top)
        file
      (condition-case ()
 	 (substring (expand-file-name file)
-		    (length 
+		    (length
 		     (expand-file-name
 		      (file-name-as-directory top))))
        (error "")))
@@ -723,7 +714,7 @@ If FILE, find the \".../etc/PACKAGE\" file instead."
 		(setq dir (concat
 			   (file-name-directory
 			    (directory-file-name (car path)))
-			   "etc/" package 
+			   "etc/" package
 			   (if file "" "/"))))
 	       (or file (file-directory-p dir)))
 	  (setq result dir
@@ -792,13 +783,13 @@ find-file-hooks, etc.
      (goto-char (point-min))
      (while (,(if regexp 're-search-forward 'search-forward)
 	     ,from nil t)
-       (insert-buffer-substring 
+       (insert-buffer-substring
 	cur start (prog1 (match-beginning 0) (set-buffer new)))
        (goto-char (point-max))
        ,(when to `(insert ,to))
        (set-buffer cur)
        (setq start (point)))
-     (insert-buffer-substring 
+     (insert-buffer-substring
       cur start (prog1 (point-max) (set-buffer new)))
      (copy-to-buffer cur (point-min) (point-max))
      (kill-buffer (current-buffer))
